#include <iostream>
#include <vector>

using namespace std;

typedef long long ll;

struct NumberInfo {
public:
    int power;
    ll remainder; 
};

struct TreeVertex {
public:
    int answer;
    vector<int> result_sequence{};
};

template<class T>
struct VectorHolder {
public:
    int is_on_heap = false;
    vector<T>* data = NULL;

    void free() {
        if (is_on_heap) {
            delete data;
            data = NULL;
        }
    }
};

int numbers_count;
NumberInfo* initial_numbers;
TreeVertex* tree;

NumberInfo decompose_power(ll number) {
    int power = 0;

    while ((number & 0b1) == 0) {
        power += 1;
        number >>= 1;
    }

    return { power, number };
}

void get_merged_result(
    vector<int>& left,
    vector<int>& right,
    vector<int>& result_sequence,
    int& result_answer,
    int lower_index,
    int upper_index
) {
    int left_index = 0;
    int right_index = 0;
    int length = left.size() + right.size();

    NumberInfo prev_number = { .power = -1, .remainder = -1 };
    int curr_sequence = 1;
    int longest_sequence = 1;

    // cout << "merging:\n";
    // for (auto i : left) cout << initial_numbers[i].power << "," << initial_numbers[i].remainder << " ";
    // cout << "\n";
    // for (auto i : right) cout << initial_numbers[i].power << "," << initial_numbers[i].remainder << " ";
    // cout << "\n";

    auto process_new_number = [lower_index, upper_index](
        int index,
        NumberInfo number,
        NumberInfo& prev_number,
        int& curr_sequence,
        int& longest_sequence,
        vector<int>& result_sequence
    ) {
        if (index < lower_index || index >= upper_index) {
            return;
        }

        if (prev_number.power == -1) {
            // do nothing!
        } else if (number.remainder != prev_number.remainder || number.power > prev_number.power + 1) {
            if (curr_sequence > longest_sequence) {
                longest_sequence = curr_sequence;
            }

            curr_sequence = 1;
        } else if (number.power == prev_number.power + 1) {
            curr_sequence += 1;
        }

        prev_number = number;
        result_sequence.push_back(index);
        // cout << initial_numbers[index].power << "," << initial_numbers[index].remainder << " ";
    };

    for (int i = 0; i < length; i++) {
        NumberInfo left_num = initial_numbers[left[left_index]];
        NumberInfo right_num = initial_numbers[right[right_index]];

        bool left_goes_first = left_num.remainder != right_num.remainder
            ? (left_num.remainder < right_num.remainder)
            : (left_num.power < right_num.power);

        if (left_goes_first) {
            process_new_number(left[left_index], left_num, prev_number, curr_sequence, longest_sequence, result_sequence);
            left_index += 1;

            if (left_index >= left.size()) {
                for (int i = right_index; i < right.size(); i++) {
                    process_new_number(right[i], initial_numbers[right[i]], prev_number, curr_sequence, longest_sequence, result_sequence);
                }

                break;
            }
        } else {
            process_new_number(right[right_index], right_num, prev_number, curr_sequence, longest_sequence, result_sequence);
            right_index += 1;

            if (right_index >= right.size()) {
                for (int i = left_index; i < left.size(); i++) {
                    process_new_number(left[i], initial_numbers[left[i]], prev_number, curr_sequence, longest_sequence, result_sequence);
                }

                break;
            }
        }
    }

    if (curr_sequence > longest_sequence) {
        longest_sequence = curr_sequence;
    }

    // cout << "\nlongest: " << longest_sequence << "\n";

    result_answer = longest_sequence - 1;
}

void init_tree(int vertex, int lower, int upper) {
    if (lower == upper - 1) {
        tree[vertex] = { .answer = 0, .result_sequence = vector<int> { lower } };
        return;
    }

    int middle = (lower + upper) / 2;

    init_tree(vertex * 2 + 1, lower, middle);
    init_tree(vertex * 2 + 2, middle, upper);

    get_merged_result(
        tree[vertex * 2 + 1].result_sequence,
        tree[vertex * 2 + 2].result_sequence,
        tree[vertex].result_sequence,
        tree[vertex].answer,
        lower,
        upper
    );

    // cout << "init " << vertex << " from " << lower << " to " << upper << ": " << tree[vertex * 2 + 1].result_sequence.size() << ", "  << tree[vertex * 2 + 2].result_sequence.size() << ", " << tree[vertex].result_sequence.size() << ", " << tree[vertex].answer << "\n";
}

int answer_query(int vertex, int lower, int upper, int query_lower, int query_upper, VectorHolder<int>& result_sequence) {
    // cout << "answer for " << vertex << " from " << lower << " to " << upper << " from " << query_lower << " to " << query_upper << "\n";
    if (query_lower >= upper || query_upper <= lower) {
    // cout << "answer for " << vertex << " from " << lower << " to " << upper << " from " << query_lower << " to " << query_upper << ": out of range\n";
        result_sequence.data = NULL;
        result_sequence.is_on_heap = false;

        return -1;
    }

    if (lower == query_lower && upper == query_upper) {
    // cout << "answer for " << vertex << " from " << lower << " to " << upper << " from " << query_lower << " to " << query_upper << ": exact " << tree[vertex].answer << "\n";
        result_sequence.is_on_heap = false;
        result_sequence.data = &tree[vertex].result_sequence;

        return tree[vertex].answer;
    }

    int middle = (lower + upper) / 2;

    VectorHolder<int> left_sequence{};
    VectorHolder<int> right_sequence{};

    int left_answer = answer_query(vertex * 2 + 1, lower, middle, query_lower, middle, left_sequence);
    int right_answer = answer_query(vertex * 2 + 2, middle, upper, middle, query_upper, right_sequence);
    // cout << "answer for " << vertex << " from " << lower << " to " << upper << " from " << query_lower << " to " << query_upper << ": answers " << left_answer << ", " << right_answer << "\n";

    if (left_answer == -1) {
        result_sequence = right_sequence;
        return right_answer;
    } else if (right_answer == -1) {
        result_sequence = left_sequence;
        return left_answer;
    }

    result_sequence.is_on_heap = true;
    result_sequence.data = new vector<int>{};

    int answer;

    get_merged_result(*left_sequence.data, *right_sequence.data, *result_sequence.data, answer, query_lower, query_upper);

    left_sequence.free();
    right_sequence.free();

    return answer;
}

int answer_query(int query_lower, int query_upper) {
    VectorHolder<int> sequence{};

    int answer = answer_query(0, 0, numbers_count, query_lower, query_upper, sequence);
    sequence.free();

    return answer;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);

    int queries_count;
    cin >> numbers_count >> queries_count;

    initial_numbers = new NumberInfo[numbers_count];
    tree = new TreeVertex[numbers_count * 4];

    for (int i = 0; i < numbers_count; i++) {
        ll number;
        cin >> number;

        initial_numbers[i] = decompose_power(number);
    }

    init_tree(0, 0, numbers_count);

    for (int query = 0; query < queries_count; query++) {
        int lower, upper;
        cin >> lower >> upper;

        lower -= 1;

        cout << answer_query(lower, upper) << "\n";
    }

    return 0;
}
